/* 
 * Bartosz Jankowski - J1TIZ
 * bjankows@stud.elka.pw.edu.pl
 * biblioteka stl, opis algorytmu : set_difference
 * File: set_difference2.cpp
 */

/* =====set_difference===== 
 * Roznica pomiedzy dwoma posortowanymi ciagami
 * Innymi slowy znajduje elty znajdujace sie w ciagu1 a nie znajdujace sie w ciagu2 i
 * zapisuje je do podanej zmiennej result. 
 */

/* =====QUICK_QUIDE=====
 * Aby zaprezentowac jego dzialanie wyobrazmy sobie prosta mini gre:
 * Mamy 2 postaci: gracza i straznika, kazda z postaci ma swoje przedmioty/ekwipunek.
 * Straznik nie przepusci gracza dopoki ten nie bedzie mial wszystkich wymaganych przedmiotow.
 * Do sprawdzenia czy gracz ma juz wszystkie 'magiczne' przedmioty wykorzystamy set_difference.
 * GOTO->Przyklad 1
 */

/* =====Parametry=====
 *
 * OutputIterator set_difference ( InputIterator1 first1, InputIterator1 last1,
 *                                 InputIterator2 first2, InputIterator2 last2,
 *                                OutputIterator result )
 *
 * OutputIterator set_difference ( InputIterator1 first1, InputIterator1 last1,
 *                                 InputIterator2 first2, InputIterator2 last2,
 *                                 OutputIterator result, Compare comp );
 *
 * first1, last1  iteratory do pierwszego i ostatniego elementu ciagu1. Przedzial: [first1,last1)  
 * first2, last2 - iteratory do pierwszego i ostatniego elementu ciagu2. Przedzial: [first2,last2)
 * result  elementy wystepujace w ciagu1 a nie wystepujace w ciagu2
 * comp - oznacza obiekt lub funkcje,ktora zostanie wykorzystana do porownywanie elementow
 *		(zamiast '<' w Algorytm) podczas  sortowania. Argumentami tej funkcji sa  
 *		dwa elementy ze zbioru tego samego typu. Zwraca ona true, jesli pierwszy element
 *		ma sie znalezc przed drugim i false w przeciwnym wypadku.(GOTO->Przyklad 3)	
 */


/* =====Zwracana wartosc=====
 * Iterator na koniec stworzonego zakresu/ciagu.  
 */

/* =====Algorytm=====

  while (first1!=last1 && first2!=last2)
   {
		if (*first1<*first2) *result++ = *first1++;
		else if (*first2<*first1) first2++;
		else { first1++; first2++; }

   }
  return copy(first1,last1,result);
 */


/* =====Zlozonosc=====
 * W najgorszym przypadku wykonuje 2*(count1+count2)-1 porownan;
 * gdzie countX jest liczba elementow pomiedzy firstX a lastX.
 */

/*
 * =====Przyklad 1; mini-game=====
 */
#include <iostream>
#include <string>
#include <vector>

/*
 * Tu znajduje sie m.in set_difference
 */
#include <algorithm>

using namespace std;

/*
 * Klasa reprezentuje postac w grze
 */
class GameCharacter				
{
public:

	/*
	 * Przedmioty ktore postac ma w ekwipunku. (vector->see STL reference)
	 */
	vector<string> items;
	GameCharacter(){}
	~GameCharacter(){}

	/*
	 * Metoda dodaje przedmiot do ekwipunku postaci
	 */
	void addItem(string item)
	{
		items.push_back(item);
	}

};

int main () {
//***************INICJALIZUJEMY EKWIPUNKI POSTACI****************
GameCharacter GateKeeper;										//Postac w grze Straznik Bramy
GateKeeper.addItem("Dragon Key");								//
GateKeeper.addItem("Silver Key");								// Przedmioty, ktore nalezy posiadac,
GateKeeper.addItem("Red Orb");									// aby straznik otworzyl brame.
GateKeeper.addItem("Vampire Fang");								//

GameCharacter Player;											//Postac w grze gracz
Player.addItem("Dragon Key");									//
Player.addItem("Bannana");										//
Player.addItem("Wine");											// 
Player.addItem("Red Orb");										//Ekwipunek gracza
Player.addItem("Helmet");										//
Player.addItem("Shield");										//
Player.addItem("Sword");										//

vector<string> missingItems(GateKeeper.items.size());			//Vector przedmiotow ktorych gracz nie 
																//posiada do przejscia przez brame 
vector<string>::iterator b = missingItems.begin();				//iterator na pierwszy przedmiot 


sort(GateKeeper.items.begin(), GateKeeper.items.end());			//Sortujemy przedmioty Straznika
sort(Player.items.begin(), Player.items.end());					//oraz gracza

//*************NIE MAMY WSZYSTKICH PRZEDMIOTOW***************
vector<string>::iterator it;									//it bedzie wskazywal na ostatni przedmiot
																//ktorego gracz nie posiada
it=set_difference (GateKeeper.items.begin(), GateKeeper.items.end(),
				   Player.items.begin(), Player.items.end(), missingItems.begin());


cout<<"Gate Keeper says: You CANNOT pass!\nYou need items: ";									
while( b!=it)													//Wypisz przedmioty ktore gracz musi zdobyc,  								
	cout <<*(b++)<<" ";											//aby przejsc przez brame tj.
																//Silver Key, Vampire Fang
missingItems.clear();

//*************ZDOBYWAMY POTRZEBNE PRZEDMIOTY****************
Player.addItem("Silver Key");									//
Player.addItem("Vampire Fang");									//Player zdobywa potrzebne przedmioty

sort(Player.items.begin(),Player.items.end());

it=set_difference (GateKeeper.items.begin(), GateKeeper.items.end(),//iterator nie wskazuje na zaden element,
				   Player.items.begin(), Player.items.end(), missingItems.begin());//missingItems jest pusty

if(missingItems.empty())										//Straznik nas przepusci:)
	cout<<"\n\nGate Keeper says: You CAN pass!\nYou miss " 
	<< (int)(it-missingItems.begin()) << " elements\n";			//0 elements
															
  return EXIT_SUCCESS;						
}

/* 
 * =====Przyklad 2===== 

 int first[] = {5,10,15,20,25};
 int second[] = {50,40,30,20,10};
  vector<int> result(10);   // 0  0  0  0  0  0  0  0  0  0
  vector<int>::iterator it;

  sort (first,first+5);     //  5 10 15 20 25
  sort (second,second+5);   // 10 20 30 40 50

  it=set_difference (first, first+5, second, second+5, result.begin());
                            // 5 15 25  0  0  0  0  0  0  0
 */

/* 
 * =====Przyklad 3; funkcja comp=====
	
	bool checkStatsLevel(Player p1, Player p2) 
	{ 
		if (p1.level < p2.level) return true;
		else return false; 
	}
 */